searchData={"items":[{"type":"module","title":"Bookk.Account","doc":"An Account is pretty much like a bucked. It has a single purpose:\nholding a measurable amount of something, in this case it's\ncurrency.","ref":"Bookk.Account.html"},{"type":"module","title":"Related - Bookk.Account","doc":"- `Bookk.AccountHead`;\n- `Bookk.Operation`;\n- `Bookk.Ledger`.","ref":"Bookk.Account.html#module-related"},{"type":"function","title":"Bookk.Account.new/2","doc":"Creates a new account from a `Bookk.AccountHead`.","ref":"Bookk.Account.html#new/2"},{"type":"function","title":"Examples - Bookk.Account.new/2","doc":"If no initial balance is provided in the second argument, then balance will be\nset to zero:\n\n    iex> head = fixture_account_head(:cash)\n    iex> Bookk.Account.new(head)\n    %Bookk.Account{\n      head: fixture_account_head(:cash),\n      balance: 0\n    }\n\nIf an initial balance is provided in the second argument, then balance will be\nset to it:\n\n    iex> head = fixture_account_head(:cash)\n    iex> Bookk.Account.new(head, 50_00)\n    %Bookk.Account{\n      head: fixture_account_head(:cash),\n      balance: 50_00\n    }","ref":"Bookk.Account.html#new/2-examples"},{"type":"function","title":"Bookk.Account.post/2","doc":"Calculates de delta amount for the operation then adds it the account's\nbalance. See `Bookk.Operation.to_delta_amount/1` for more information on\ndelta amount.","ref":"Bookk.Account.html#post/2"},{"type":"function","title":"Examples - Bookk.Account.post/2","doc":"iex> class = %Bookk.AccountClass{natural_balance: :debit}\n    iex> head = %Bookk.AccountHead{class: class}\n    iex> account = Bookk.Account.new(head)\n    iex>\n    iex> op = debit(head, 25_00)\n    iex>\n    iex> Bookk.Account.post(account, op)\n    %Bookk.Account{\n      head: %Bookk.AccountHead{class: %Bookk.AccountClass{natural_balance: :debit}},\n      balance: 25_00\n    }\n\nThe account's head must match the head in the operation, otherwise an error is\nraised:\n\n    iex> head_a = %Bookk.AccountHead{name: \"a\"}\n    iex> head_b = %Bookk.AccountHead{name: \"b\"}\n    iex>\n    iex> account = Bookk.Account.new(head_a)\n    iex> op = debit(head_b, 25_00)\n    iex>\n    iex> Bookk.Account.post(account, op)\n    ** (FunctionClauseError) no function clause matching in Bookk.Account.post/2","ref":"Bookk.Account.html#post/2-examples"},{"type":"type","title":"Bookk.Account.t/0","doc":"The struct that represents the state of an account.","ref":"Bookk.Account.html#t:t/0"},{"type":"type","title":"Fields - Bookk.Account.t/0","doc":"An account is composed of:\n- `head`: the `Bookk.AccountHead` that identifies the account;\n- `balance`: the amount of currency held by the account, in cents or\n  the smallest fraction supported by the currency you're using.","ref":"Bookk.Account.html#t:t/0-fields"},{"type":"module","title":"Bookk.AccountClass","doc":"An account class has properties that are inherited by accounts and\nserves as a way of grouping account's balances.","ref":"Bookk.AccountClass.html"},{"type":"module","title":"Common classes - Bookk.AccountClass","doc":"Here are some of the most common used classes for reference:\n\n| id       | parent id | natural balance | name                     |\n| ---      | ---       | ---             | ---                      |\n| **`A`**  |           | `:debit`        | Assets                   |\n| **`CA`** | **`A`**   | `:debit`        | Current Assets           |\n| **`AR`** | **`A`**   | `:debit`        | Accounts Receivables     |\n| **`Ac`** |           | `:credit`       | Contra Assets            |\n| **`AD`** | **`Ac`**  | `:credit`       | Accumulated Depreciation |\n| **`E`**  |           | `:debit`        | Expenses                 |\n| **`OE`** |           | `:credit`       | Owner's Equity           |\n| **`L`**  |           | `:credit`       | Liabilities              |\n| **`AP`** | **`L`**   | `:credit`       | Accounts Payables        |\n| **`I`**  |           | `:credit`       | Income                   |\n| **`G`**  | **`I`**   | `:credit`       | Gains                    |\n| **`R`**  | **`I`**   | `:credit`       | Revenue                  |","ref":"Bookk.AccountClass.html#module-common-classes"},{"type":"module","title":"Related - Bookk.AccountClass","doc":"- `Bookk.AccountHead`;\n- `Bookk.Account`.","ref":"Bookk.AccountClass.html#module-related"},{"type":"type","title":"Bookk.AccountClass.t/0","doc":"The struct that describes an account class.","ref":"Bookk.AccountClass.html#t:t/0"},{"type":"type","title":"Fields - Bookk.AccountClass.t/0","doc":"An account class in composed of:\n- **`id`**: it's recomended that, instead of using it with an\n  arbitrary value, you assign it to the class' name abbreviation;\n- **`parent_id`**: If the class is a subclass, then `parent_id`\n  should be set to the parent class' abbreviation. For example,\n  Current Assets is a subclass of Assets, therefore its `parent_id`\n  should be set to `\"A\"` (where `\"A\"` is the abbreviation of\n  Assets);\n- **`name`**: The human readable name of the account class;\n- **`natural_balance`** (either `:debit` or `:credit`): specifies\n  the direction in which accounts of this class grows their balance.\n  For example, Assets accounts grows their balances with `:debit`\n  operations.\n\nSee section [Common classes](#module-common-classes) for examples of\nclasses.","ref":"Bookk.AccountClass.html#t:t/0-fields"},{"type":"module","title":"Bookk.AccountHead","doc":"An Account head is a struct contained all the values necessary to\neither fetch or create a `Bookk.Account` from/into a `Bookk.Ledger`.","ref":"Bookk.AccountHead.html"},{"type":"module","title":"Related - Bookk.AccountHead","doc":"- `Bookk.AccountClass`;\n- `Bookk.Account`;\n- `Bookk.Ledger`;\n- `Bookk.Operations`.","ref":"Bookk.AccountHead.html#module-related"},{"type":"type","title":"Bookk.AccountHead.t/0","doc":"The struct that describes an account head.","ref":"Bookk.AccountHead.html#t:t/0"},{"type":"type","title":"Fields - Bookk.AccountHead.t/0","doc":"An account head is composed of:\n- `name`: the accounts name (unique within a ledger);\n- `class`: a `Bookk.AccountClass` struct that describes the class to\n  which the account belongs;\n- `meta`: a map of metadata for whatever information you find useful\n  to hold there for when you need to fetch, create or \"hydrate\" a\n  persisted account.","ref":"Bookk.AccountHead.html#t:t/0-fields"},{"type":"behaviour","title":"Bookk.ChartOfAccounts","doc":"A Chart of Accounts (abbrv.: CoA) is a mapping of all the accounts\nand all the ledgers that can exist in your system. But instead of\nhard-coding them, you define patterns for accounts and ledgers\nsupported by your application using functions and pattern matching.\n\nFor example, if your application allows ledgers to have an account\nfor exampenses from paying salary to an employee, you could define a\nfunction with a signature the like the one below:\n\n    def account({:salary, {:employee, employee_id}})\n\nAnd if your application, following the previous example, allows for\nevery employee to have their own ledger, you could define a function\nwith a signature like the one below:\n\n    def ledger({:employee, employee_id})","ref":"Bookk.ChartOfAccounts.html"},{"type":"behaviour","title":"Related - Bookk.ChartOfAccounts","doc":"- `Bookk.Notation`;\n- `Bookk.AccountClass`;\n- `Bookk.AccountHead`.","ref":"Bookk.ChartOfAccounts.html#module-related"},{"type":"callback","title":"Bookk.ChartOfAccounts.account/1","doc":"This function maps all possible patterns of accounts that your\napplication supports. It's recomended to use pattern matching and\nlet it crash in the event of a mismatch.","ref":"Bookk.ChartOfAccounts.html#c:account/1"},{"type":"callback","title":"Examples - Bookk.ChartOfAccounts.account/1","doc":"def account(:cash), do: %Bookk.AccountHead{...}\n    def account(:deposits), do: %Bookk.AccountHead{...}\n    def account({:payables, {:user, id}}), do: %Bookk.AccountHead{...}\n    def account({:receivables, {:user, id}}), do: %Bookk.AccountHead{...}","ref":"Bookk.ChartOfAccounts.html#c:account/1-examples"},{"type":"callback","title":"Bookk.ChartOfAccounts.ledger/1","doc":"This function maps all possible patterns of ledger names your\napplication supports. It's recomended to use pattern matching and\nlet it crash in the event of a mismatch.","ref":"Bookk.ChartOfAccounts.html#c:ledger/1"},{"type":"callback","title":"Example - Bookk.ChartOfAccounts.ledger/1","doc":"def ledger(:acme), do: \"acme\"\n    def ledger({:user,  >}), do: \"user(#{id})\"","ref":"Bookk.ChartOfAccounts.html#c:ledger/1-example"},{"type":"module","title":"Bookk.InterledgerEntry","doc":"An interledger entry is a collection of journal entries affecting\nmultiple ledgers that must be transacted under the same accounting\ntransaction. It's somewhat analogous to an `Ecto.Multi` holding\nmultiple operations or a git commit that affect multiple files.","ref":"Bookk.InterledgerEntry.html"},{"type":"module","title":"Related - Bookk.InterledgerEntry","doc":"- `Bookk.Notation`;\n- `Bookk.NaiveState`;\n- `Bookk.JournalEntry`.","ref":"Bookk.InterledgerEntry.html#module-related"},{"type":"function","title":"Bookk.InterledgerEntry.balanced?/1","doc":"Checks whether the interledger entry is balanced. It is balance if\nall of its journal entries are balanced.","ref":"Bookk.InterledgerEntry.html#balanced?/1"},{"type":"function","title":"Examples - Bookk.InterledgerEntry.balanced?/1","doc":"Balanced entry:\n\n    iex> interledger = %Bookk.InterledgerEntry{\n    iex>   entries_by_ledger: %{\n    iex>     \"acme\" => [\n    iex>       %Bookk.JournalEntry{\n    iex>         operations: [\n    iex>           fixture_account_head(:cash) |> debit(30_00),\n    iex>           fixture_account_head(:deposits) |> credit(30_00)\n    iex>         ]\n    iex>       }\n    iex>     ]\n    iex>   }\n    iex> }\n    iex>\n    iex> Bookk.InterledgerEntry.balanced?(interledger)\n    true\n\nUnbalanced entry:\n\n    iex> interledger = %Bookk.InterledgerEntry{\n    iex>   entries_by_ledger: %{\n    iex>     \"acme\" => [\n    iex>       %Bookk.JournalEntry{\n    iex>         operations: [\n    iex>           fixture_account_head(:cash) |> debit(30_00),\n    iex>         ]\n    iex>       }\n    iex>     ]\n    iex>   }\n    iex> }\n    iex>\n    iex> Bookk.InterledgerEntry.balanced?(interledger)\n    false","ref":"Bookk.InterledgerEntry.html#balanced?/1-examples"},{"type":"function","title":"Bookk.InterledgerEntry.empty?/1","doc":"Checks whether an interledger entry is empty. It is empty when it\nhas now journal entries or when all its journal entries are empty.\n\nSee `Bookk.JournalEntry.empty?/1` to learn more about empty journal\nentries.","ref":"Bookk.InterledgerEntry.html#empty?/1"},{"type":"function","title":"Examples - Bookk.InterledgerEntry.empty?/1","doc":"Is empty when there's no entries:\n\n    iex> Bookk.InterledgerEntry.empty?(%Bookk.InterledgerEntry{})\n    true\n\nIs empty when all entries are empty:\n\n    iex> interledger = %Bookk.InterledgerEntry{\n    iex>   entries_by_ledger: %{\n    iex>     \"acme\" => [\n    iex>       %Bookk.JournalEntry{\n    iex>         operations: [%Bookk.Operation{amount: 0}]\n    iex>       }\n    iex>     ]\n    iex>   }\n    iex> }\n    iex>\n    iex> Bookk.InterledgerEntry.empty?(interledger)\n    true\n\nIs not empty when at least one entry isn't empty:\n\n    iex> interledger = %Bookk.InterledgerEntry{\n    iex>   entries_by_ledger: %{\n    iex>     \"acme\" => [\n    iex>       %Bookk.JournalEntry{\n    iex>         operations: [\n    iex>           %Bookk.Operation{amount: 0},\n    iex>           %Bookk.Operation{amount: 1},\n    iex>         ]\n    iex>       }\n    iex>     ]\n    iex>   }\n    iex> }\n    iex>\n    iex> Bookk.InterledgerEntry.empty?(interledger)\n    false","ref":"Bookk.InterledgerEntry.html#empty?/1-examples"},{"type":"function","title":"Bookk.InterledgerEntry.reverse/1","doc":"Produces a new interledger entry that is equaly opposite of the\ngiven interledger entry, meaning its capable of reverting all the\nchanges that the given entry causes.","ref":"Bookk.InterledgerEntry.html#reverse/1"},{"type":"function","title":"Examples - Bookk.InterledgerEntry.reverse/1","doc":"Reverses all of its journal entries:\n\n    iex> interledger = %Bookk.InterledgerEntry{\n    iex>   entries_by_ledger: %{\n    iex>     \"acme\" => [\n    iex>       %Bookk.JournalEntry{\n    iex>         operations: [\n    iex>           fixture_account_head(:cash) |> debit(10_00),\n    iex>           fixture_account_head(:deposits) |> credit(10_00)\n    iex>         ]\n    iex>       }\n    iex>     ]\n    iex>   }\n    iex> }\n    iex>\n    iex> Bookk.InterledgerEntry.reverse(interledger)\n    %Bookk.InterledgerEntry{\n      entries_by_ledger: %{\n        \"acme\" => [\n          %Bookk.JournalEntry{\n            operations: [\n              fixture_account_head(:deposits) |> debit(10_00),\n              fixture_account_head(:cash) |> credit(10_00)\n            ]\n          }\n        ]\n      }\n    }","ref":"Bookk.InterledgerEntry.html#reverse/1-examples"},{"type":"function","title":"Bookk.InterledgerEntry.to_journal_entries/1","doc":"Given an interledger entry, it returns all its journal entries in\nthe form of a list of tuples where the first element is the ledger's\nname and the second element is a list of journal entries that are\nmeant to be posted to such ledger.","ref":"Bookk.InterledgerEntry.html#to_journal_entries/1"},{"type":"function","title":"Examples - Bookk.InterledgerEntry.to_journal_entries/1","doc":"Returns a list of tuple where the first element is the ledger name\nand the second element is a journal entry:\n\n    iex> user_id = \"b13a81cf-ff78-414d-b5b2-042e9ecf2082\"\n    iex> cash = fixture_account_head(:cash)\n    iex> deposits = fixture_account_head(:deposits)\n    iex> unspent_cash = fixture_account_head({:unspent_cash, {:user, user_id}})\n    iex>\n    iex> interledger = %Bookk.InterledgerEntry{\n    iex>   entries_by_ledger: %{\n    iex>     \"acme\" => [\n    iex>       Bookk.JournalEntry.new([\n    iex>         debit(cash, 50_00),\n    iex>         credit(unspent_cash, 50_00)\n    iex>       ])\n    iex>     ],\n    iex>     \"user(b13a81cf-ff78-414d-b5b2-042e9ecf2082)\" => [\n    iex>       Bookk.JournalEntry.new([\n    iex>         debit(cash, 50_00),\n    iex>         credit(deposits, 50_00)\n    iex>       ])\n    iex>     ]\n    iex>   }\n    iex> }\n    iex>\n    iex> Bookk.InterledgerEntry.to_journal_entries(interledger)\n    [\n      {\"acme\", Bookk.JournalEntry.new([\n        debit(fixture_account_head(:cash), 50_00),\n        credit(fixture_account_head({:unspent_cash, {:user, \"b13a81cf-ff78-414d-b5b2-042e9ecf2082\"}}), 50_00)\n      ])},\n      {\"user(b13a81cf-ff78-414d-b5b2-042e9ecf2082)\", Bookk.JournalEntry.new([\n        debit(fixture_account_head(:cash), 50_00),\n        credit(fixture_account_head(:deposits), 50_00)\n      ])}\n    ]","ref":"Bookk.InterledgerEntry.html#to_journal_entries/1-examples"},{"type":"type","title":"Bookk.InterledgerEntry.t/0","doc":"The struct that represents an interledger entry.","ref":"Bookk.InterledgerEntry.html#t:t/0"},{"type":"type","title":"Fields - Bookk.InterledgerEntry.t/0","doc":"An interledger entry is composed of:\n- `entries_by_ledger`: the map of journal entries that are included\n  in the interledger entry, grouped by the name of the ledger\n  against which they should be posted.","ref":"Bookk.InterledgerEntry.html#t:t/0-fields"},{"type":"module","title":"Bookk.JournalEntry","doc":"A Journal Entry is a set of operations that must be transacted under\nthe same accounting transaction. Those operations describe a change\nin balance for an account. Operations are analogous to git commits,\nthey represent a diff on a account's balance.","ref":"Bookk.JournalEntry.html"},{"type":"module","title":"Related - Bookk.JournalEntry","doc":"- `Bookk.Ledger`;\n- `Bookk.Operation`;\n- `Bookk.AccountHead`.","ref":"Bookk.JournalEntry.html#module-related"},{"type":"function","title":"Bookk.JournalEntry.balanced?/1","doc":"Checks whether a journal entry is balanced. It is considered balance\nwhen the sum of its debit operations is equal the sum of its credit\noperations.","ref":"Bookk.JournalEntry.html#balanced?/1"},{"type":"function","title":"Examples - Bookk.JournalEntry.balanced?/1","doc":"Is balanced when the sum of debits is equal the sum of credits:\n\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     fixture_account_head(:cash) |> debit(10_00),\n    iex>     fixture_account_head(:deposits) |> credit(10_00)\n    iex>   ]\n    iex> }\n    iex>\n    iex> Bookk.JournalEntry.balanced?(journal_entry)\n    true\n\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     fixture_account_head(:cash) |> debit(10_00),\n    iex>     fixture_account_head(:deposits) |> credit(7_00),\n    iex>     fixture_account_head(:deposits) |> credit(3_00),\n    iex>   ]\n    iex> }\n    iex>\n    iex> Bookk.JournalEntry.balanced?(journal_entry)\n    true\n\nIs unbalanced when the sum of debits isn't equal the sum of credits:\n\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     fixture_account_head(:cash) |> debit(10_00)\n    iex>   ]\n    iex> }\n    iex>\n    iex> Bookk.JournalEntry.balanced?(journal_entry)\n    false","ref":"Bookk.JournalEntry.html#balanced?/1-examples"},{"type":"function","title":"Bookk.JournalEntry.empty?/1","doc":"Checks whether a journal entry is empty.","ref":"Bookk.JournalEntry.html#empty?/1"},{"type":"function","title":"Examples - Bookk.JournalEntry.empty?/1","doc":"Is empty when the journal entry has no operations:\n\n    iex> Bookk.JournalEntry.empty?(%Bookk.JournalEntry{})\n    true\n\nIs empty when all operations in the journal entry are empty:\n\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     %Bookk.Operation{amount: 0}\n    iex>   ]\n    iex> }\n    iex>\n    iex> Bookk.JournalEntry.empty?(journal_entry)\n    true\n\nIs not empty when at least one operation in the journal entry isn't\nempty:\n\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     %Bookk.Operation{amount: 10_00}\n    iex>   ]\n    iex> }\n    iex>\n    iex> Bookk.JournalEntry.empty?(journal_entry)\n    false","ref":"Bookk.JournalEntry.html#empty?/1-examples"},{"type":"function","title":"Bookk.JournalEntry.new/1","doc":"Creates a new journal entry from a set of operations.","ref":"Bookk.JournalEntry.html#new/1"},{"type":"function","title":"Examples - Bookk.JournalEntry.new/1","doc":"If there're multiple operations touching the same account, they will\nbe merged into a single operation:\n\n    iex> cash = fixture_account_head(:cash)\n    iex> deposits = fixture_account_head(:deposits)\n    iex>\n    iex> Bookk.JournalEntry.new([\n    iex>   debit(cash, 80_00),\n    iex>   debit(cash, 20_00),\n    iex>   credit(deposits, 100_00)\n    iex> ])\n    %Bookk.JournalEntry{\n      operations: [\n        fixture_account_head(:cash) |> debit(100_00),\n        fixture_account_head(:deposits) |> credit(100_00)\n      ]\n    }","ref":"Bookk.JournalEntry.html#new/1-examples"},{"type":"function","title":"Bookk.JournalEntry.reverse/1","doc":"Creates a new journal entry that reverses all effects from the given\njournal entry.","ref":"Bookk.JournalEntry.html#reverse/1"},{"type":"function","title":"Examples - Bookk.JournalEntry.reverse/1","doc":"Reverses all operations in the journal entry:\n\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     fixture_account_head(:cash) |> debit(10_00),\n    iex>     fixture_account_head(:deposits) |> credit(10_00)\n    iex>   ]\n    iex> }\n    iex>\n    iex> Bookk.JournalEntry.reverse(journal_entry)\n    %Bookk.JournalEntry{\n      operations: [\n        fixture_account_head(:deposits) |> debit(10_00),\n        fixture_account_head(:cash) |> credit(10_00)\n      ]\n    }","ref":"Bookk.JournalEntry.html#reverse/1-examples"},{"type":"function","title":"Bookk.JournalEntry.to_operations/1","doc":"Returns the list of operations inside a journal entry.","ref":"Bookk.JournalEntry.html#to_operations/1"},{"type":"function","title":"Examples - Bookk.JournalEntry.to_operations/1","doc":"Returns the journal entry's list of operations:\n\n    iex> Bookk.JournalEntry.new([\n    iex>   debit(fixture_account_head(:cash), 50_00),\n    iex>   credit(fixture_account_head(:deposits), 50_00)\n    iex> ])\n    iex> |> Bookk.JournalEntry.to_operations()\n    [\n      debit(fixture_account_head(:cash), 50_00),\n      credit(fixture_account_head(:deposits), 50_00)\n    ]","ref":"Bookk.JournalEntry.html#to_operations/1-examples"},{"type":"type","title":"Bookk.JournalEntry.t/0","doc":"The struct that describe a Journal Entry.","ref":"Bookk.JournalEntry.html#t:t/0"},{"type":"type","title":"Fields - Bookk.JournalEntry.t/0","doc":"- `operations`: the list of operations included in the journal entry.","ref":"Bookk.JournalEntry.html#t:t/0-fields"},{"type":"module","title":"Bookk.Ledger","doc":"A ledger is a book that holds accounts. Traditionally, ledgers would\nalso hold the journal entries that changed the accounts but, in this\nlibrary, persisting those journal entries is considered off scope.\nYou may persist state the way the best fits your needs.","ref":"Bookk.Ledger.html"},{"type":"module","title":"Related - Bookk.Ledger","doc":"- `Bookk.Account`;\n- `Bookk.JournalEntry`.","ref":"Bookk.Ledger.html#module-related"},{"type":"function","title":"Bookk.Ledger.balanced?/1","doc":"Checks whether the ledger is balanced.\n\nA ledger is considered balance when the some of balance from its\ndebit accounts is equal the sum of balance from its credit accounts.\nYou know if an account is a \"debit account\" or a \"credit account\" by\nthe natural balance of its class.\n\nSee `Bookk.AccountClass` for more information on natural balance.","ref":"Bookk.Ledger.html#balanced?/1"},{"type":"function","title":"Examples - Bookk.Ledger.balanced?/1","doc":"Is balanced when the ledger is empty:\n\n    iex> Bookk.Ledger.new(\"acme\")\n    iex> |> Bookk.Ledger.balanced?()\n    true\n\nIs balanced when the sum of debit accounts balances is equal the sum\nof credit accounts balances:\n\n    iex> ledger = Bookk.Ledger.new(\"acme\")\n    iex> cash = fixture_account_head(:cash)\n    iex> deposits = fixture_account_head(:deposits)\n    iex>\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     debit(cash, 50_00),\n    iex>     credit(deposits, 50_00)\n    iex>   ]\n    iex> }\n    iex>\n    iex> Bookk.Ledger.post(ledger, journal_entry)\n    iex> |> Bookk.Ledger.balanced?()\n    true\n\nIs unbalanced when the sum of debit accounts balances isn't equal\nthe sum of credit accounts balances:\n\n    iex> ledger = Bookk.Ledger.new(\"acme\")\n    iex> cash = fixture_account_head(:cash)\n    iex>\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     debit(cash, 50_00)\n    iex>   ]\n    iex> }\n    iex>\n    iex> Bookk.Ledger.post(ledger, journal_entry)\n    iex> |> Bookk.Ledger.balanced?()\n    false","ref":"Bookk.Ledger.html#balanced?/1-examples"},{"type":"function","title":"Bookk.Ledger.get_account/2","doc":"Get an account from the ledger by its `Bookk.AccountHead`. If the\naccount doesn't exist yet, then an account will be returned with\nempty state.","ref":"Bookk.Ledger.html#get_account/2"},{"type":"function","title":"Examples - Bookk.Ledger.get_account/2","doc":"Returns the account when it exists in the ledger:\n\n    iex> ledger = %Bookk.Ledger{\n    iex>   name: \"acme\",\n    iex>   accounts_by_name: %{\n    iex>     \"cash/CA\" => %Bookk.Account{\n    iex>       head: fixture_account_head(:cash),\n    iex>       balance: 25_00\n    iex>     }\n    iex>   }\n    iex> }\n    iex>\n    iex> Bookk.Ledger.get_account(ledger, fixture_account_head(:cash))\n    %Bookk.Account{\n      head: fixture_account_head(:cash),\n      balance: 25_00\n    }\n\nReturns an empty account when the it doesn't exist in the ledger:\n\n    iex> Bookk.Ledger.new(\"acme\")\n    iex> |> Bookk.Ledger.get_account(fixture_account_head(:cash))\n    %Bookk.Account{\n      head: fixture_account_head(:cash),\n      balance: 0\n    }","ref":"Bookk.Ledger.html#get_account/2-examples"},{"type":"function","title":"Bookk.Ledger.new/2","doc":"Creates a new `Bookk.Ledger` from its name and, optionally, a list\nof `Bookk.Account`.","ref":"Bookk.Ledger.html#new/2"},{"type":"function","title":"Bookk.Ledger.post/2","doc":"Posts a `Bookk.JournalEntry` to a ledger. This means that the\nbalance change described in each operation of the journal entry will\nbe applied to their respective accounts of the ledger. If there's a\nchange to an account that doesn't exist yet, then the account is\nfirst created.","ref":"Bookk.Ledger.html#post/2"},{"type":"function","title":"Examples - Bookk.Ledger.post/2","doc":"When account doesn't exist then it gets created:\n\n    iex> ledger = Bookk.Ledger.new(\"acme\")\n    iex>\n    iex> cash = fixture_account_head(:cash)\n    iex> deposits = fixture_account_head(:deposits)\n    iex>\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     debit(cash, 50_00),\n    iex>     credit(deposits, 50_00)\n    iex>   ]\n    iex> }\n    iex>\n    iex> updated_ledger = Bookk.Ledger.post(ledger, journal_entry)\n    iex>\n    iex> %Bookk.Account{balance: 50_00} = Bookk.Ledger.get_account(updated_ledger, cash)\n    iex> %Bookk.Account{balance: 50_00} = Bookk.Ledger.get_account(updated_ledger, deposits)\n\nWhen account exists then it gets updated:\n\n    iex> ledger = Bookk.Ledger.new(\"acme\")\n    iex>\n    iex> cash = fixture_account_head(:cash)\n    iex> deposits = fixture_account_head(:deposits)\n    iex>\n    iex> journal_entry = %Bookk.JournalEntry{\n    iex>   operations: [\n    iex>     debit(cash, 50_00),\n    iex>     credit(deposits, 50_00)\n    iex>   ]\n    iex> }\n    iex>\n    iex> updated_ledger =\n    iex>   ledger\n    iex>   |> Bookk.Ledger.post(journal_entry)\n    iex>   |> Bookk.Ledger.post(journal_entry) # post twice\n    iex>\n    iex> %Bookk.Account{balance: 100_00} = Bookk.Ledger.get_account(updated_ledger, cash)\n    iex> %Bookk.Account{balance: 100_00} = Bookk.Ledger.get_account(updated_ledger, deposits)","ref":"Bookk.Ledger.html#post/2-examples"},{"type":"type","title":"Bookk.Ledger.t/0","doc":"The struct that represents a ledger.","ref":"Bookk.Ledger.html#t:t/0"},{"type":"type","title":"Fields - Bookk.Ledger.t/0","doc":"- `name`: the name of the ledger;\n- `accounts_by_name`: a map of the accounts known by the ledger,\n  grouped by their name.","ref":"Bookk.Ledger.html#t:t/0-fields"},{"type":"module","title":"Bookk.NaiveState","doc":"A state struct that holds multiple ledgers. It's considered \"naive\"\nbecause it doesn't hold any information regarding the journal\nentries that put the state into its current value.","ref":"Bookk.NaiveState.html"},{"type":"module","title":"Related - Bookk.NaiveState","doc":"- `Bookk.Notation`;\n- `Bookk.InterledgerEntry`;\n- `Bookk.Ledger`.","ref":"Bookk.NaiveState.html#module-related"},{"type":"function","title":"Bookk.NaiveState.empty/0","doc":"Produces a empty naive state.","ref":"Bookk.NaiveState.html#empty/0"},{"type":"function","title":"Bookk.NaiveState.get_ledger/2","doc":"Get's a ledger from the state by its name. If the ledger doesn't\nexist in the state yet, then a new empty ledger will be returned.","ref":"Bookk.NaiveState.html#get_ledger/2"},{"type":"function","title":"Examples - Bookk.NaiveState.get_ledger/2","doc":"Returns an empty ledger when requested ledger doesn't exist in\nstate:\n\n    iex> Bookk.NaiveState.get_ledger(%Bookk.NaiveState{}, \"acme\")\n    %Bookk.Ledger{name: \"acme\"}\n\nReturns the ledger when it exists in state:\n\n    iex> state = %Bookk.NaiveState{\n    iex>   ledgers_by_name: %{\n    iex>     \"foo\" => %Bookk.Ledger{\n    iex>       accounts_by_name: %{\n    iex>         \"cash\" => %Bookk.Account{}\n    iex>       }\n    iex>     }\n    iex>   }\n    iex> }\n    iex>\n    iex> Bookk.NaiveState.get_ledger(state, \"foo\")\n    %Bookk.Ledger{\n      accounts_by_name: %{\n        \"cash\" => %Bookk.Account{}\n      }\n    }","ref":"Bookk.NaiveState.html#get_ledger/2-examples"},{"type":"function","title":"Bookk.NaiveState.new/1","doc":"Produces a new state struct from a set of ledgers.","ref":"Bookk.NaiveState.html#new/1"},{"type":"function","title":"Bookk.NaiveState.post/2","doc":"Posts a `Bookk.InterledgerEntry` to the state, appling changes in\nbalance to multiple accounts accross multiple ledgers.","ref":"Bookk.NaiveState.html#post/2"},{"type":"function","title":"Examples - Bookk.NaiveState.post/2","doc":"iex> import Bookk.Notation, only: [journalize!: 2]\n    iex>\n    iex> user_id = \"123\"\n    iex> deposited_amount = 500_00\n    iex>\n    iex> journal_entry =\n    iex>   journalize! using: TestChartOfAccounts do\n    iex>     on ledger(:acme) do\n    iex>       debit account(:cash), deposited_amount\n    iex>       credit account({:unspent_cash, {:user, user_id}}), deposited_amount\n    iex>     end\n    iex>\n    iex>     on ledger({:user, user_id}) do\n    iex>       debit account(:cash), deposited_amount\n    iex>       credit account(:deposits), deposited_amount\n    iex>     end\n    iex>   end\n    iex>\n    iex> Bookk.NaiveState.empty()\n    iex> |> Bookk.NaiveState.post(journal_entry)\n    %Bookk.NaiveState{\n      ledgers_by_name: %{\n        \"acme\" => %Bookk.Ledger{\n          name: \"acme\",\n          accounts_by_name: %{\n            fixture_account_head(:cash).name => %Bookk.Account{\n              head: fixture_account_head(:cash),\n              balance: 500_00\n            },\n            fixture_account_head({:unspent_cash, {:user, \"123\"}}).name => %Bookk.Account{\n              head: fixture_account_head({:unspent_cash, {:user, \"123\"}}),\n              balance: 500_00\n            }\n          }\n        },\n        \"user(123)\" => %Bookk.Ledger{\n          name: \"user(123)\",\n          accounts_by_name: %{\n            fixture_account_head(:cash).name => %Bookk.Account{\n              head: fixture_account_head(:cash),\n              balance: 500_00\n            },\n            fixture_account_head(:deposits).name => %Bookk.Account{\n              head: fixture_account_head(:deposits),\n              balance: 500_00\n            }\n          }\n        }\n      }\n    }","ref":"Bookk.NaiveState.html#post/2-examples"},{"type":"type","title":"Bookk.NaiveState.t/0","doc":"The struct representing a naive state.","ref":"Bookk.NaiveState.html#t:t/0"},{"type":"type","title":"Fields - Bookk.NaiveState.t/0","doc":"- `ledgers_by_name`: the ledgers known by the state, grouped by\n  their name.","ref":"Bookk.NaiveState.html#t:t/0-fields"},{"type":"module","title":"Bookk.Notation","doc":"DSL notation for describing an interledger entries\n(`Bookk.InterledgerEntry`).","ref":"Bookk.Notation.html"},{"type":"module","title":"Related - Bookk.Notation","doc":"- `Bookk.ChartOfAccounts`;\n- `Bookk.InterledgerEntry`;\n- `Bookk.NaiveState`.","ref":"Bookk.Notation.html#module-related"},{"type":"macro","title":"Bookk.Notation.journalize/2","doc":"DSL notation for describing an interledger entries\n(`Bookk.InterledgerEntry`).","ref":"Bookk.Notation.html#journalize/2"},{"type":"macro","title":"Examples - Bookk.Notation.journalize/2","doc":"Returns a balanced interledger journal entry:\n\n    iex> import Bookk.Notation, only: [journalize: 2]\n    iex>\n    iex> %Bookk.InterledgerEntry{} = journal_entry =\n    iex>   journalize using: TestChartOfAccounts do\n    iex>     on ledger(:acme) do\n    iex>       debit account(:cash), 150_00\n    iex>       credit account(:deposits), 150_00\n    iex>     end\n    iex>   end\n    iex>\n    iex> assert not Bookk.InterledgerEntry.empty?(journal_entry)\n    iex> assert Bookk.InterledgerEntry.balanced?(journal_entry)\n\nReturns an unbalanced interledger journal entry:\n\n    iex> import Bookk.Notation, only: [journalize: 2]\n    iex>\n    iex> %Bookk.InterledgerEntry{} = journal_entry =\n    iex>   journalize using: TestChartOfAccounts do\n    iex>     on ledger(:acme) do\n    iex>       debit account(:cash), 150_00\n    iex>       credit account(:deposits), 50_00\n    iex>     end\n    iex>   end\n    iex>\n    iex> assert not Bookk.InterledgerEntry.empty?(journal_entry)\n    iex> assert not Bookk.InterledgerEntry.balanced?(journal_entry)","ref":"Bookk.Notation.html#journalize/2-examples"},{"type":"macro","title":"Bookk.Notation.journalize!/2","doc":"Same as `journalize/2` but it raises an error if the resulting\ninterledger journal entry is unbalanced.","ref":"Bookk.Notation.html#journalize!/2"},{"type":"macro","title":"Examples - Bookk.Notation.journalize!/2","doc":"Returns a balanced interledger journal entry:\n\n    iex> import Bookk.Notation, only: [journalize!: 2]\n    iex>\n    iex> %Bookk.InterledgerEntry{} = journal_entry =\n    iex>   journalize! using: TestChartOfAccounts do\n    iex>     on ledger(:acme) do\n    iex>       debit account(:cash), 150_00\n    iex>       credit account(:deposits), 150_00\n    iex>     end\n    iex>   end\n    iex>\n    iex> assert not Bookk.InterledgerEntry.empty?(journal_entry)\n    iex> assert Bookk.InterledgerEntry.balanced?(journal_entry)\n\nRaises an error when an unbalanced interledger journal entry is\nproduced:\n\n    iex> import Bookk.Notation, only: [journalize!: 2]\n    iex>\n    iex> journalize! using: TestChartOfAccounts do\n    iex>   on ledger(:acme) do\n    iex>     debit account(:cash), 150_00\n    iex>     credit account(:deposits), 50_00\n    iex>   end\n    iex> end\n    ** (Bookk.UnbalancedError) `journalize!/2` produced an unbalanced journal entry","ref":"Bookk.Notation.html#journalize!/2-examples"},{"type":"module","title":"Bookk.Operation","doc":"An operation describe a change in balance on a single account\n(`Bookk.Account`).","ref":"Bookk.Operation.html"},{"type":"module","title":"Related - Bookk.Operation","doc":"- `Bookk.JournalEntry`;\n- `Bookk.AccountHead`;\n- `Bookk.AccountClass`.","ref":"Bookk.Operation.html#module-related"},{"type":"function","title":"Bookk.Operation.credit/2","doc":"Creates a credit operation from a `Bookk.AccountHead` and an amount.","ref":"Bookk.Operation.html#credit/2"},{"type":"function","title":"Examples - Bookk.Operation.credit/2","doc":"Crediting a positive amount produces a credit operation:\n\n    iex> head = fixture_account_head(:cash)\n    iex> Bookk.Operation.credit(head, 25_00)\n    %Bookk.Operation{\n      direction: :credit,\n      account_head: fixture_account_head(:cash),\n      amount: 25_00\n    }\n\nCrediting a negative amount produces a debit operation:\n\n    iex> head = fixture_account_head(:cash)\n    iex> Bookk.Operation.credit(head, -25_00)\n    %Bookk.Operation{\n      direction: :debit,\n      account_head: fixture_account_head(:cash),\n      amount: 25_00\n    }","ref":"Bookk.Operation.html#credit/2-examples"},{"type":"function","title":"Bookk.Operation.debit/2","doc":"Creates a debit operation given a `Bookk.AccountHead` and an amount.","ref":"Bookk.Operation.html#debit/2"},{"type":"function","title":"Examples - Bookk.Operation.debit/2","doc":"Debiting a positive amount produces a debit operation:\n\n    iex> head = fixture_account_head(:cash)\n    iex> Bookk.Operation.debit(head, 25_00)\n    %Bookk.Operation{\n      direction: :debit,\n      account_head: fixture_account_head(:cash),\n      amount: 25_00\n    }\n\nDebiting a negative amount produces a credit operation:\n\n    iex> head = fixture_account_head(:cash)\n    iex> Bookk.Operation.debit(head, -25_00)\n    %Bookk.Operation{\n      direction: :credit,\n      account_head: fixture_account_head(:cash),\n      amount: 25_00\n    }","ref":"Bookk.Operation.html#debit/2-examples"},{"type":"function","title":"Bookk.Operation.empty?/1","doc":"Checks whether an operation is empty. It is considered empty when\nits amount is zero, meaning no changes to the account's balance.","ref":"Bookk.Operation.html#empty?/1"},{"type":"function","title":"Examples - Bookk.Operation.empty?/1","doc":"Is empty when amount is zero:\n\n    iex> Bookk.Operation.empty?(%Bookk.Operation{amount: 0})\n    true\n\nIs not empty when amount is different than zero:\n\n    iex> Bookk.Operation.empty?(%Bookk.Operation{amount: 1})\n    false","ref":"Bookk.Operation.html#empty?/1-examples"},{"type":"function","title":"Bookk.Operation.merge/1","doc":"Same as {Bookk.Operation.merge/2} but it takes a non-empty list of\noperations, all to the same account (same `account_head`).","ref":"Bookk.Operation.html#merge/1"},{"type":"function","title":"Examples - Bookk.Operation.merge/1","doc":"iex> head = fixture_account_head(:cash)\n    iex>\n    iex> a = debit(head, 100_00)\n    iex> b = debit(head, 200_00)\n    iex> c = debit(head, 300_00)\n    iex>\n    iex> Bookk.Operation.merge([a, b, c])\n    %Bookk.Operation{\n      direction: :debit,\n      account_head: fixture_account_head(:cash),\n      amount: 600_00\n    }\n\nIf an empty list is provided, then an error will be raised:\n\n    iex> Bookk.Operation.merge([])\n    ** (FunctionClauseError) no function clause matching in Bookk.Operation.merge/1","ref":"Bookk.Operation.html#merge/1-examples"},{"type":"function","title":"Bookk.Operation.merge/2","doc":"Combines two operation against the same account into one operation.","ref":"Bookk.Operation.html#merge/2"},{"type":"function","title":"Examples - Bookk.Operation.merge/2","doc":"When the two operations have the same direction, the resulting\noperation has the same direction and the sum of the two amounts as\nits amount:\n\n    iex> head = fixture_account_head(:cash)\n    iex>\n    iex> a = debit(head, 70_00)\n    iex> b = debit(head, 30_00)\n    iex>\n    iex> Bookk.Operation.merge(a, b)\n    %Bookk.Operation{\n      direction: :debit,\n      account_head: fixture_account_head(:cash),\n      amount: 100_00\n    }\n\nWhen the two operations have different direction, the account's\nnatural balance will define the resulting direction. As for the\nresulting amount, the operation that matches the natural balance\ndirection will have its amount subtracted by the other operation's\namount:\n\n    iex> head = fixture_account_head(:cash)\n    iex>\n    iex> a = debit(head, 70_00)\n    iex> b = credit(head, 30_00)\n    iex>\n    iex> Bookk.Operation.merge(a, b)\n    %Bookk.Operation{\n      direction: :debit,\n      account_head: fixture_account_head(:cash),\n      amount: 40_00\n    }\n\nIf the resulting balance is a negative number, then the resulting\ndirection will be switched and the amount will be transformed into a\npositive number:\n\n    iex> head = fixture_account_head(:cash)\n    iex>\n    iex> a = credit(head, 70_00)\n    iex> b = debit(head, 30_00)\n    iex>\n    iex> Bookk.Operation.merge(a, b)\n    %Bookk.Operation{\n      direction: :credit,\n      account_head: fixture_account_head(:cash),\n      amount: 40_00\n    }\n\nIf the operations' account heads aren't the same in both operations,\nthen an error will be raised:\n\n    iex> a = debit(fixture_account_head(:cash), 10_00)\n    iex> b = credit(fixture_account_head(:deposits), 10_00)\n    iex>\n    iex> Bookk.Operation.merge(a, b)\n    ** (FunctionClauseError) no function clause matching in Bookk.Operation.merge/2","ref":"Bookk.Operation.html#merge/2-examples"},{"type":"function","title":"Bookk.Operation.new/3","doc":"Creates a new operation. Same as `credit/2` and `debit/2` but the\noperation's direction is provided as an atom argument.","ref":"Bookk.Operation.html#new/3"},{"type":"function","title":"Bookk.Operation.reverse/1","doc":"Produces a opposite operation from the given operation. The opposite\noperation is capable of reverting the effect of the given operation\nwhen posted to an account.","ref":"Bookk.Operation.html#reverse/1"},{"type":"function","title":"Examples - Bookk.Operation.reverse/1","doc":"A credit operation becomes a debit operation:\n\n    iex> entry = %Bookk.Operation{direction: :credit, amount: 10_00}\n    iex> Bookk.Operation.reverse(entry)\n    %Bookk.Operation{direction: :debit, amount: 10_00}\n\nA debit operation becomes a credit operation:\n\n    iex> entry = %Bookk.Operation{direction: :debit, amount: 10_00}\n    iex> Bookk.Operation.reverse(entry)\n    %Bookk.Operation{direction: :credit, amount: 10_00}","ref":"Bookk.Operation.html#reverse/1-examples"},{"type":"function","title":"Bookk.Operation.to_delta_amount/1","doc":"Returns the operation's delta amount, which is the real number\n(positive or negative integer) by which the account's balance will\nbe changed.\n\nA negative integer is return in case the account should be\nsubtracted, making this value safe to always be used with an\naddition operation against the account's balance.","ref":"Bookk.Operation.html#to_delta_amount/1"},{"type":"function","title":"Examples - Bookk.Operation.to_delta_amount/1","doc":"Debiting an account which has a debit natural balance produces a\npositive number:\n\n    iex> head = %Bookk.AccountHead{\n    iex>   class: %Bookk.AccountClass{natural_balance: :debit}\n    iex> }\n    iex>\n    iex> debit(head, 100_00)\n    iex> |> Bookk.Operation.to_delta_amount()\n    100_00\n\nDebiting an account which has a credit natural balance produces a\nnegative number:\n\n    iex> head = %Bookk.AccountHead{\n    iex>   class: %Bookk.AccountClass{natural_balance: :debit}\n    iex> }\n    iex>\n    iex> credit(head, 100_00)\n    iex> |> Bookk.Operation.to_delta_amount()\n    -100_00\n\nCrediting an account which has a credit natural balance produces a\npositive number:\n\n    iex> head = %Bookk.AccountHead{\n    iex>   class: %Bookk.AccountClass{natural_balance: :credit}\n    iex> }\n    iex>\n    iex> credit(head, 100_00)\n    iex> |> Bookk.Operation.to_delta_amount()\n    100_00\n\nDebiting an account which has a credit natural balance produces a\nnegative number:\n\n    iex> head = %Bookk.AccountHead{\n    iex>   class: %Bookk.AccountClass{natural_balance: :credit}\n    iex> }\n    iex>\n    iex> debit(head, 100_00)\n    iex> |> Bookk.Operation.to_delta_amount()\n    -100_00","ref":"Bookk.Operation.html#to_delta_amount/1-examples"},{"type":"function","title":"Bookk.Operation.uniq/1","doc":"Takes a set of operations and returns a set of uniq operations per\naccount. The operations that affect the same account will be merged.\n\nSee `merge/1` and `merge/2` for more information on merging\noperations.","ref":"Bookk.Operation.html#uniq/1"},{"type":"function","title":"Examples - Bookk.Operation.uniq/1","doc":"When there's more than one operation touching the same account,\nthose operations are merged together so that the resulting list\ncontains one a single operation thouching each account:\n\n    iex> cash = fixture_account_head(:cash)\n    iex> deposits = fixture_account_head(:deposits)\n    iex>\n    iex> a = debit(cash, 40_00)\n    iex> b = debit(cash, 60_00)\n    iex> c = credit(deposits, 100_00)\n    iex>\n    iex> Bookk.Operation.uniq([a, b, c])\n    [\n      fixture_account_head(:cash) |> debit(100_00),\n      fixture_account_head(:deposits) |> credit(100_00)\n    ]\n\nWhen all operations are unique, the list is returned as is:\n\n    iex> cash = fixture_account_head(:cash)\n    iex> deposits = fixture_account_head(:deposits)\n    iex>\n    iex> a = debit(cash, 100_00)\n    iex> b = credit(deposits, 100_00)\n    iex>\n    iex> Bookk.Operation.uniq([a, b])\n    [\n      fixture_account_head(:cash) |> debit(100_00),\n      fixture_account_head(:deposits) |> credit(100_00)\n    ]\n\nWhen an empty list is given, the result will also be an empty list:\n\n    iex> Bookk.Operation.uniq([])\n    []","ref":"Bookk.Operation.html#uniq/1-examples"},{"type":"type","title":"Bookk.Operation.t/0","doc":"The struct representing an operation.","ref":"Bookk.Operation.html#t:t/0"},{"type":"type","title":"Fields - Bookk.Operation.t/0","doc":"- `direction` (either `:debit` or `:credit`): by itself it means\n  nothing -- it's just a lable -- but, once combined with the\n  account's natural balance (`account_head.class.natural_balance`),\n  then we're able to tell if the operation will result in an\n  addition or a subtraction of balance;\n- `account_head`: a `Bookk.AccountHead` struct used to either\n  update or create the affected account in the ledger where the\n  operation was posted;\n- `amount`: the [positive] amount by which the account's balance\n  will be changed. Whether the change will be an addition or a\n  subtraction, that depends on `direction` and the account class'\n  natural balance.","ref":"Bookk.Operation.html#t:t/0-fields"},{"type":"exception","title":"Bookk.UnbalancedError","doc":"An error representing that something (a journal entry, a ledger, a\nstate...) isn't balanced.","ref":"Bookk.UnbalancedError.html"},{"type":"extras","title":"Bookk","doc":"# Bookk\n\nBookk is a simple library that provides building blocks for operating journal entries and manipulating double-entry bookkeeping accounting ledgers.\n\nSee full documentation at [hexdocs](https://hexdocs.pm/bookk).\n\nThis library aims to decrease the friction between domain specialists (mainly accountants) and developers by providing a DSL that enables developers to write code for journal entries with a syntax that's familiar to specialists. That way, it should be easy for specialists to review code for journal entries and, likewise, it whould be easy for developers to implement journal entries based on instructions provided by specialists.\n\n```elixir\ndef to_interledger_entry(%Deposit{} = tx) do\n  journalize! using: ACME.ChartOfAccounts do\n    on ledger(:acme) do\n      debit account(:cash), tx.amount\n      credit account({:unspent_cash, {:user, tx.user_id}}), tx.amount\n    end\n\n    on ledger({:user, tx.user_id}) do\n      debit account(:cash), tx.amount\n      credit account(:deposits), tx.amount\n    end\n  end\nend\n```\n_(A journal entry from a deposit operation affecting two ledgers written with\nBookk's DSL)_\n\nPersisting state, such as accounts balances and log of transactions per accounts is considered off scope for this library at the moment — and honestly, might never becomes part of its scope — but you can still do it own your own. An example of how to persist state using `Ecto` is provided in section **Examples** at [Persist State using Ecto](#persist-state-using-ecto).\n\nVisit page **API Reference** for a brief introduction to double-entry bookkeeping concepts implemented by this library.","ref":"readme.html"},{"type":"extras","title":"Installation - Bookk","doc":"The package can be installed by adding `bookk` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:bookk, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Examples - Bookk","doc":"> **Warning**\n> The snipets in this section are meant to be taken as a pseudocode, they haven't been tested yet.\n\n#","ref":"readme.html#examples"},{"type":"extras","title":"Chart of Accounts - Bookk","doc":"```elixir\ndefmodule ACME.ChartOfAccounts do\n  @behaviour Bookk.ChartOfAccounts\n\n  alias Bookk.AccountClass, as: C\n  alias Bookk.AccountHead, as: H\n\n  # some of the most common account classes\n  @classes %{\n     A: %C{id:  \"A\", parent_id: nil, natural_balance:  :debit, name: \"Assets\"},\n    CA: %C{id: \"CA\", parent_id: \"A\", natural_balance:  :debit, name: \"Current Assets\"},\n    AR: %C{id: \"AR\", parent_id: \"A\", natural_balance:  :debit, name: \"Accounts Receivables\"},\n     E: %C{id:  \"E\", parent_id: nil, natural_balance:  :debit, name: \"Expenses\"},\n    OE: %C{id: \"OE\", parent_id: nil, natural_balance: :credit, name: \"Owner's Equity\"},\n     L: %C{id:  \"L\", parent_id: nil, natural_balance: :credit, name: \"Liabilities\"},\n    AP: %C{id: \"AP\", parent_id: \"L\", natural_balance: :credit, name: \"Accounts Payables\"},\n     I: %C{id:  \"I\", parent_id: nil, natural_balance: :credit, name: \"Income\"},\n     G: %C{id:  \"G\", parent_id: \"I\", natural_balance: :credit, name: \"Gains\"},\n     R: %C{id:  \"R\", parent_id: \"I\", natural_balance: :credit, name: \"Revenue\"}\n  }\n\n  @impl Bookk.ChartOfAccounts\n  def ledger(:acme), do: \"acme\"\n  def ledger({:user,  >}), do: \"user(#{id})\"\n\n  @impl Bookk.ChartOfAccounts\n  def account(:cash), do: %H{name: \"cash/CA\", class: @classes.CA}\n  def account(:deposits), do: %H{name: \"deposits/OE\", class: @classes.OE}\n  def account({:unspent_cash, {:user,  >}}), do: %H{name: \"unspent-cash:user(#{id})/L\", class: @classes.L}\n  def account({:deposit_expenses,  >}), do: %H{name: \"deposit-expenses:#{provider}/E\", class: @classes.E}\n\n  @doc false\n  def account_id(ledger_name, %H{} = account_head), do: ledger_name   \":\"   account_head.name\nend\n```\n\n### (DSL) Interledger Entry\n\nHere's demonstrated how to journalize (create a journal entry template, if you may) for an specific accounting transaction. Examples are using `ACME.ChartOfAccounts` from the previous section.\n\n**When user deposits balance (using a single ledger):**\n\n```elixir\nimport Bookk.Notation, only: [journalize!: 2]\n\ninterledger_entry =\n  journalize! using: ACME.ChartOfAccounts do\n    on ledger(:acme) do\n      debit account(:cash), deposited_amount\n      credit account({:unspent_cash, {:user, user_id}}), deposited_amount\n    end\n  end\n\nupdated_state =\n  Bookk.NaiveState.empty()\n  |> Bookk.NaiveState.post(interledger_entry)\n```\n\n**When user deposited balance (using multiple ledgers):**\n\n```elixir\nimport Bookk.Notation, only: [journalize!: 2]\n\ninterledger_entry =\n  journalize! using: ACME.ChartOfAccounts do\n    on ledger(:acme) do\n      debit account(:cash), deposited_amount\n      credit account({:unspent_cash, {:user, user_id}}), deposited_amount\n    end\n\n    on ledger({:user, user_id}) do\n      debit account(:cash), deposited_amount\n      credit account(:deposits), deposited_amount\n    end\n  end\n\nupdated_state =\n  Bookk.NaiveState.empty()\n  |> Bookk.NaiveState.post(interledger_entry)\n```\n\n\n#","ref":"readme.html#chart-of-accounts"},{"type":"extras","title":"Persist state using Ecto - Bookk","doc":"This section demostrantes how state can be persisted to a database using `Ecto` instead of posting (apply side-effects) to the in-memory structs provided by the library (such as `Bookk.Ledger` and `Bookk.NaiveState`).\n\nFirst, we'll need two models.\n\n1.  **Account**, which holds the account's balance:\n\n    ```elixir\n    defmodule Account do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      @primary_key false\n      schema \"accounts\" do\n        field :id, :string, primary_key: true\n        field :ledger_id, :string\n        field :balance, :integer\n        field :created_at, :utc_datetime_usec\n        field :updated_at, :utc_datetime_usec\n      end\n\n      @doc false\n      @spec changeset(t, map) :: Ecto.Changeset.t()\n\n      def changeset(account \\\\ %__MODULE__{}, %{} = fields) do\n        account\n        |> cast(fields, [:id, :ledger_id, :balance, :created_at, :udpated_at])\n        |> validate_required([:id, :ledger_id, :balance, :created_at, :udpated_at])\n      end\n    end\n    ```\n\n2.  **AccountTransaction**, which serves as a log of changes to an accounts:\n\n    ```elixir\n    defmodule AccountTransaction do\n      use Ecto.Schema\n\n      import Ecto.Changeset\n\n      @primary_key false\n      schema \"accounts_transactions\" do\n        field :account_id, :string, primary_key: true\n        field :transaction_id, Ecto.UUID, primary_key: true\n        field :delta_amount, :integer\n        field :balance_after, :integer\n        field :created_at, :utc_datetime_usec\n      end\n\n      @doc false\n      @spec changeset(t, map) :: Ecto.Changeset.t()\n\n      def changeset(account_transfer \\\\ %__MODULE__{}, %{} = fields) do\n        account_transfer\n        |> cast(fields, [:account_id, :transaction_id, :delta_amount, :balance, :created_at])\n        |> validate_required([:account_id, :transaction_id, :delta_amount, :balance, :created_at])\n      end\n    end\n    ```\n\nWe'll also have a `Transactionable` protocol that specifies what functions are\nexpected from structs that describe side effects to our accounting system:\n\n```elixir\ndefprotocol Transactionable do\n  @moduledoc false\n\n  @typedoc false\n  @type t :: %{\n              required(__struct__) => atom,\n              required(:id) => String.t(),\n              optional(atom) => any\n            }\n\n  @doc false\n  @spec to_interledger_entry(t) :: Bookk.InterledgerEntry.t()\n\n  def to_interledger_entry(tx)\nend\n```\n\nNow we have a `DepositTransaction` describing that a user deposited balance into\ntheir account — note that it implements `Transactionable` protocol:\n\n```elixir\ndefmodule DepositTransaction do\n  @moduledoc false\n\n  @typedoc false\n  @type t :: %DepositTransaction{\n              id: String.t(),\n              user_id: String.t(),\n              amount: pos_integer\n            }\n\n  defstruct [:id, :user_id, :amount]\nend\n\ndefimpl Transactionable, for: DepositTransaction do\n  use Bookk.Notation\n\n  @impl Transactionable\n  def to_interledger_entry(tx) do\n    journalize! using: ACME.ChartOfAccounts do\n      on ledger(:acme) do\n        debit account(:cash), tx.amount\n        credit account({:unspent_cash, {:user, tx.user_id}})\n      end\n\n      on ledger({:user, tx.user_id}) do\n        debit account(:cash), tx.amount\n        credit account(:deposits), tx.amount\n      end\n    end\n  end\nend\n```\n\nAnd finally, we have our `Accounting` module that knows how to take a `Transactionable` struct and persist its side-effects to the database using `Ecto`:\n\n```elixir\ndefmodule Accounting do\n  @moduledoc false\n\n  @doc false\n  @spec transact(Transactionable.t()) :: {:ok, Ecto.Multi.t()} | {:error, term}\n\n  def transact(tx) do\n    interledger_entry = Transactionable.to_interledger_entry(tx)\n    now = DateTime.utc_now()\n\n    multis =\n      for {ledger_name, journal_entry}   Enum.reduce(Ecto.Multi.new(), &Ecto.Multi.append(&2, &1))\n    |> ACME.Repo.transaction()\n  end\n\n  defp op_to_multi(%Bookk.Operation{} = op, ledger_name, tx_id, now) do\n    #   we need uniq names for each multi operation, there will be 2 of them for\n    # ↓ each `Bookk.Operation`\n    multi_a_name = Ecto.UUID.generate()\n    multi_b_name = Ecto.UUID.generate()\n\n    #   the amount by which the account's balance should change (either a\n    #   positive or negative integer — in cents or the smallest fraction of the\n    # ↓ currency you're using)\n    delta_amount = Bookk.Operations.to_delta_amount(op)\n    account_id = ACME.ChartOfAccounts.account_id(ledger_name, op.account_head)\n\n    account_changeset =\n      Account.changeset(%{\n        id: account_id,\n        ledger_id: ledger_name,\n        balance: delta_amount,\n        created_at: now,\n        updated_at: now\n      })\n\n    Ecto.Multi.new()\n    #  ↓ upserts the account\n    |> Ecto.Multi.insert(multi_a_name, account_changeset, [\n      conflict_target: :id,\n      on_conflict: [\n        inc: [balance: delta_amount],\n        set: [updated_at: now]\n      ],\n      returning: [:balance]\n    ])\n    #    creates the log entry recording the amount by which the account's\n    #  ↓ balance changed in this accounting transaction\n    |> Ecto.Multi.insert(multi_b_name, fn %{^multi_a_name => updated_accocunt} ->\n      AccountTransaction.changeset(%{\n        account_id: account_id,\n        transaction_id: tx_id,\n        delta_amount: delta_amount,\n        balance_after: updated_account.balance,\n        created_at: now\n      })\n    end)\n  end\nend\n```","ref":"readme.html#persist-state-using-ecto"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2023 Rafael Willians\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown"}